package bighornSheep75;
/*$$
 * Copyright (c) 1999, Trustees of the University of Chicago
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE TRUSTEES OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *$$*/

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StreamTokenizer;

import javax.swing.JOptionPane;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.space.Dimensions;
import repast.simphony.ui.RSApplication;
import repast.simphony.valueLayer.ValueLayer;


/**
 * A class that represents a raster image as a space.  This is designed to be
 * used in conjunction with a GIS system such as ESRI ArcGis or GRASS.
 * The raster image is treated as a continuous system of coordinates with
 * cells that hold values underlying.  Much of the code for reading the
 * ESRI ASCII Raster format is based on code by <a href:"http://www.geog.leeds.ac.uk/staff/i.turnton/i.turton.html">Ian Turton</a> Centre for Computational
 * Geography University of Leeds, LS2 9Jt, 1998. <BR>
 * <a href="mailto:ian.geog.leeds.ac.uk">i.turton@geog.leeds.ac.uk</a>, as 
 * included in <a href="http://geotools.sourceforge.net">GeoTools</a>, an open
 * source gis visualization framework.
 **/

public class ShortValueLayer2D implements ValueLayer {

	String name;
    int height;
    int width;
    
    public int map_x_begin;
    public int map_x_end;
    public int map_y_begin;
    public int map_y_end;
    public double map_sum;
    
	// Parameter from ESRI ASCII file
    double sparseness;
    double originx;
    double originy;
    double termx;
    double termy;
    double cellSize;
    double nodata;
    double min;
    double max;
    double missing = Double.NaN;

	
    public short[][] matrix;
	
    /**
     * Constructs an empty RasterSpace.
     * @param top The uppermost y coordinate
     * @param left The leftmost x coordinate
     * @param cellSize The size of a raster cell
     * @param h The height of the space
     * @param w The width of the space
     **/
    public ShortValueLayer2D(int w, int h, String _name) {
        name = _name;
        height = h;
        width = w;
        matrix = new short[w][h];
    }

    /**
     * Creates a RasterSpace from an ESRI ASCII Raster file.
     * @param name The name of the ESRI ASCII raster file.
     * @throws java.io.IOException if there is a problem reading the file
     **/
    public ShortValueLayer2D(String name, String path) throws IOException {
    	try {
	    	this.name = name;
	        FileInputStream stream = new FileInputStream(path);
	        init(stream,1);
	        compute_map_boundaries();
    	}
    	catch (FileNotFoundException e) {  
    		String msg = "File not found : " + path;
    		JOptionPane.showMessageDialog(RSApplication.getRSApplicationInstance().getGui().getFrame().getGlassPane(),msg,"Initialization error", JOptionPane.INFORMATION_MESSAGE);
    		RunEnvironment.getInstance().getParameters().setValue("dureeDeLaSimulation", 0);
    	}  	
    	catch (Exception e) {  
    		String msg = "An error occurs parsing the file : " + path;
    		JOptionPane.showMessageDialog(RSApplication.getRSApplicationInstance().getGui().getFrame().getGlassPane(),msg,"Initialization error", JOptionPane.INFORMATION_MESSAGE);
    		RunEnvironment.getInstance().getParameters().setValue("dureeDeLaSimulation", 0);
    	}  	
    	
    }

    /**
     * Creates a RasterSpace from an ESRI ASCII Raster file.
     * @param name The name of the ESRI ASCII raster file.
     * @throws java.io.IOException if there is a problem reading the file
     **/
    //public ShortValueLayer2D(String name, String path, boolean rescale) throws IOException {
    	//try {
	    	//this.name = name;
	       // FileInputStream stream = new FileInputStream(path);
	       // init(stream,1);
	       // stream.close();
	        
	       // if(rescale) {
	        //	FileInputStream stream2 = new FileInputStream(path);
		      //  init(stream2, 32766.0/this.max);
		        //stream2.close();
	       // }
	       // compute_map_boundaries();
    	//}
    	//catch (FileNotFoundException e) {  
    		//String msg = "File not found : " + path;
    	//	JOptionPane.showMessageDialog(RSApplication.getRSApplicationInstance().getGui().getFrame().getGlassPane(),msg,"Initialization error", JOptionPane.INFORMATION_MESSAGE);
    		//RunEnvironment.getInstance().getParameters().setValue("dureeDeLaSimulation", 0);
    //	}  	
    	//catch (Exception e) {  
    		//String msg = "An error occurs parsing the file : " + path;
    	//	JOptionPane.showMessageDialog(RSApplication.getRSApplicationInstance().getGui().getFrame().getGlassPane(),msg,"Initialization error", JOptionPane.INFORMATION_MESSAGE);
    	//	RunEnvironment.getInstance().getParameters().setValue("dureeDeLaSimulation", 0);
    //	}  	
    	
  //  }

 

	public void writeToAscii(String _path)
    {
    	try{
    		BufferedWriter out = new BufferedWriter(new FileWriter(_path, false));
    		out.write("ncols         " + String.valueOf(width));
    		out.newLine();
    		out.write("nrows         " + String.valueOf(height));
    		out.newLine();
    		out.write("xllcorner     " + String.valueOf(originx));
    		out.newLine();
    		out.write("yllcorner     " + String.valueOf(originy));
    		out.newLine();
    		out.write("cellsize      " + String.valueOf(cellSize));
    		out.newLine();
    		out.write("NODATA_value  " + String.valueOf(nodata));
    		out.newLine();   
    		
    		for(int ligne=0;ligne<height;ligne++) {
    			for(int col=0;col<width;col++) {
    				out.write(String.valueOf(matrix[col][height - ligne - 1]));
    				out.write(" ");
    			}
    			out.newLine(); 
    		}
    			
    		out.close();
    		
    	}
    	catch (Exception e) {
    		RSApplication.getRSApplicationInstance().getGui().showError("hello", "world");
    		//JOptionPane.showMessageDialog(RSApplication.getRSApplicationInstance().getGui().showError("hello", "world"),"Text Message","MessageBox Title", JOptionPane.INFORMATION_MESSAGE);
    		
    		System.out.println(e.toString());
    	}
    	
    }
    
    
    private void init(InputStream stream, double factor) throws IOException {
        int type;
        BufferedReader r = new BufferedReader(new InputStreamReader(stream));
        StreamTokenizer st = new StreamTokenizer(r);

        st.parseNumbers();
        st.wordChars('_', '_');
        st.eolIsSignificant(false);
        st.lowerCaseMode(true);
        //cols
        type = st.nextToken();
        type = st.nextToken();
        width = (int) st.nval;
        //rows
        type = st.nextToken();
        type = st.nextToken();
        height = (int) st.nval;
        //xllcorner
        type = st.nextToken();
        type = st.nextToken();
        originx = st.nval;
        //yllcorner
        type = st.nextToken();
        type = st.nextToken();
        originy = st.nval;
        //cellSize
        type = st.nextToken();
        type = st.nextToken();
        cellSize = st.nval;
        //termx and termy
        termx = Math.floor(originx) + cellSize * width;
        termy = Math.floor(originy) + cellSize * height;
        //missing
        type = st.nextToken();
        if (type == StreamTokenizer.TT_NUMBER) {
            st.pushBack();
            nodata = missing;
        } else {
            type = st.nextToken();
            nodata = st.nval;
        }
        st.ordinaryChars('E', 'E');
        st.ordinaryChar('e');

        min = Short.MAX_VALUE;
        max = Short.MIN_VALUE;
        matrix = new short[width][height];
        double d1;

        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                st.nextToken();
                d1 = st.nval;
                
                type = st.nextToken();
                if (type != StreamTokenizer.TT_NUMBER && type != StreamTokenizer.TT_EOF) {
                    st.nextToken();
                    d1 = d1 * Math.pow(10.0, st.nval);
                } else {
                    st.pushBack();
                }
                
                if(d1!=nodata) {
                	d1=d1*factor;
                	min = Math.min(min, d1);
                    max = Math.max(max, d1);
                }

                matrix[j][height - i - 1]=(short)d1; 
                // J'ai chang������ la valeur de l'index "[i]" pour "[height - i - 1]"
                // Ce qui a permis d'afficher le territoire dans le bon sens
            }
        }
        r.close();
    }

    public void compute_map_boundaries() {
        map_x_begin = 9999;
        map_x_end = -1;
        map_y_begin = 9999;
        map_y_end = -1;
        map_sum=0;
        
        for(int x=0; x<this.getSizeX(); x++) {
        	for(int y=0; y<this.getSizeY(); y++) {
        		
        		if (matrix[x][y]!=nodata) {
        			map_sum+=matrix[x][y];
        		}
        		
        		if (matrix[x][y]>0 && x<map_x_begin) {
        			map_x_begin = x;
        		}
        		
        		if (matrix[x][y]>0 && x>map_x_end) {
        			map_x_end = x;
        		}
        		
        		if (matrix[x][y]>0 && y<map_y_begin) {
        			map_y_begin = y;
        		}
        		
        		if (matrix[x][y]>0 && y>map_y_end) {
        			map_y_end = y;
        		}
        	}
        }
    }
    
    public double getOriginX() {
        return originx;
    }
    
    public double getCellSize(){
    	return cellSize;
    	
    }
    

    public double getOriginY() {
        return originy;
    }

    public double getTermX() {
        return termx;
    }

    public double getTermY() {
        return termy;
    }

    public Dimensions getDimensions() {
    	int[] dim = new int[2];
    	dim[0]=getSizeX();
    	dim[1]=getSizeY();
        return new Dimensions(dim);
    }

    public int getSizeX() {
        return width;
    }

    public int getSizeY() {
        return height;
    }
    
    public String getName() {
    	return name;
    }
    public double getnodata(){
    	return nodata;
    }

    /**
     * Get the value at an integer point.
     * @param x The column
     * @param y The row
     **/
    public short get(int x, int y) {
         if (x>=0 && x <width && y>=0 && y<height) {
            return matrix[x][y];
        } else {
            throw new IllegalArgumentException("Array OutOfBound ShortValueLayer2D.getValueAt()");
        }
    }
    
    /**
     * Get the value at an double point, please use integer !
     * @param x The column
     * @param y The row
     **/
    public double get(double... coords) {
        if (coords[0]>=0 && coords[0] <width && coords[1]>=0 && coords[1]<height) {
            return (double)matrix[(int)coords[0]][(int)coords[1]];
        } else {
            throw new IllegalArgumentException("Array OutOfBound ShortValueLayer2D.getValueAt()");
        }
    }
	
    /**
     * Put a value at an integer point.
     * @param x The column
     * @param y The row
     **/
    public void set(int x, int y, short value) {
        if (x>=0 && x<width && y>=0 && y<height) {
            matrix[x][y]=value;
        } else {
            throw new IllegalArgumentException("Array OutOfBound ShortValueLayer2D.setValueAt()");
        }
    }
    
    public void clear() {
    	for(int x=0; x<this.getSizeX(); x++) {
        	for(int y=0; y<this.getSizeY(); y++) {
        		matrix[x][y]=0;
        	}
    	}
    }
}