package bighornSheep75;
import java.io.BufferedWriter;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.Scanner;

import repast.simphony.context.Context;
import repast.simphony.context.space.continuous.ContinuousSpaceFactoryFinder;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.continuous.RandomCartesianAdder;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.RandomGridAdder;

//*************************************************************************************************
//Implements the model by building and returning a context. A spatial grid and space is created 
//and populated with agents and a projection created. There are two ways of creating agents:
//by creating a list of all the known sheep occurrences on the actual landscape (read in from 
//an ASCII text file) and creating 100 agents and placing them on a sheep occurrence location; or
//randomly placing a single sheep agent on a pixel with suitable habitat. This model and associated 
//data is designed for implementation in the Repast Simphony (version 2.2) environment.
//*************************************************************************************************
public class contextBuilder implements ContextBuilder<Object> {

	public Context<Object> build(Context<Object> context) {
		Parameters params = RunEnvironment.getInstance().getParameters();
		GIS.setup();
		int xdim = GIS.boundries.getSizeX();
		int ydim = GIS.boundries.getSizeY();

		context.addValueLayer(GIS.boundries);
	
		//creates a 2D grid space so agents know their location with respect to environmental variables.
		Grid<Object> patchDisplay = GridFactoryFinder.createGridFactory(null)
				.createGrid("Display Grid", context,
						new GridBuilderParameters<Object>(new repast.simphony.space.grid.WrapAroundBorders(),
					new RandomGridAdder<Object>(), 
						true, xdim, ydim));
		System.out.println(patchDisplay.getDimensions());
		 
		// Create a new 2D continuous space to model the physical space on which the agents
		// will move. 
		ContinuousSpace<Object> space = ContinuousSpaceFactoryFinder.createContinuousSpaceFactory(null)
		.createContinuousSpace("Continuous Space", 
				context, 
				new RandomCartesianAdder<Object>(),
				new repast.simphony.space.continuous.WrapAroundBorders(), 
				xdim,ydim);	
		
		/**creates a list of all known bighorn occurrences and places 100 agents on the 
		 * occurrence point determined from "Parameters" on the GUI.
		 */
		//**
		List<gridcellList> wetlandCells  = new ArrayList<gridcellList>();
		context.add(wetlandCells);
		for(int x = 0; x < xdim; x++)
			for(int y = 0; y < ydim; y++)
			{
				if(GIS.sheep.get(x, y) !=-9999)
				{
					gridcellList addThis = new gridcellList(x,y);
					wetlandCells.add(addThis);
				}
			}
		int wetland = (Integer)params.getValue("wetland");
		System.out.println(params.getValue("wetland"));
		int numAgents = 100;
		for(int i =0; i<numAgents; i++)
	{	
			agent agent = new agent(patchDisplay, space);
			context.add(agent);
			space.moveTo(agent, wetlandCells.get(wetland).getX(),wetlandCells.get(wetland).getY());
			patchDisplay.moveTo(agent, wetlandCells.get(wetland).getX(),wetlandCells.get(wetland).getY());
		}
		
	//**/
		
		/**
		 * creates one agent and places it on a random pixel that meets bighorn sheep habitat requirements.		
		 */
		/**
		Random random = new Random();
		int numAgents = 1;
		for(int i =0; i<numAgents; i++)
		{
			agent agent = new agent(patchDisplay, space);
			context.add(agent);
			NdPoint pt = space.getLocation(agent);
			space.moveTo(agent, pt.getX(), pt.getY());
			patchDisplay.moveTo(agent,(int) space.getLocation(agent).getX(),(int) space.getLocation(agent).getY());
			boolean foundSquare = false;
			
			while(foundSquare == false)
			{
			
				int randomX = random.nextInt(xdim) + 1;
				int randomY = random.nextInt(ydim) + 1;
				space.moveTo(agent, randomX, randomY);
				patchDisplay.moveTo(agent, randomX, randomY);
				pt=space.getLocation(agent);
				foundSquare = true;
				
				if(pt.getX()<GIS.lakes.getSizeX() && pt.getY() < GIS.lakes.getSizeY())
					if(GIS.lakes.matrix[(int) pt.getX()] [(int) pt.getY()] !=-9999)
					foundSquare = false;
				
				if(foundSquare == true)
				{
					movementList test = new movementList((int) pt.getX(), (int) pt.getY(), (int) pt.getX(),(int) pt.getY(), patchDisplay, space);
					if(test.satisfiesEscapeTerrain() == false)
						foundSquare = false;
					if(test.getCover()>40)
						foundSquare = false;
				if(test.getRiver() !=-9999)
						foundSquare = false;
					if(test.getRoad()!=-9999)
						foundSquare = false;
				}	
			}	
		}
		//**/
	//	System.out.println(params.getValue("Default Random Seed"));
		
		
		/**	
		 * Used to aggregate the output from a simulation into a single ASCII text file, which
		 * can be outputted into ArcMap.
		 */
		//**
		for(int i = 0; i<GIS.resultsLayer.getSizeX(); i++)
		for(int j = 0; j<GIS.resultsLayer.getSizeY(); j++)
		{
			GIS.resultsLayer.set(i, j, (short) 0);
		}
	
	int x, y, z, crownCover, escapeTerrain;
	try{
		Scanner scanner = new Scanner(new File("output/ModelOutput.2016.Apr.05.09_23_40.txt"));
		//scanner.useDelimiter(",");
		scanner.nextLine();
		while(scanner.hasNext())
		{
			z = scanner.nextInt();
			x = scanner.nextInt();
			y = scanner.nextInt();
			//crownCover = scanner.nextInt();
			//escapeTerrain = scanner.nextInt();
		
		if(x < GIS.resultsLayer.getSizeX() && y < GIS.resultsLayer.getSizeY())
			try{
				int value = GIS.resultsLayer.get(x,y);
				value = value +1;
				GIS.resultsLayer.set(x,y, (short) value);
				//System.out.println(GIS.resultsLayer.get(x,y));
			}catch(Exception e)
			{Exception e1;}
	}
		scanner.close();
	}catch(IOException e){
		System.out.println("didn't work");
	}
	
	try{
		BufferedWriter out = new BufferedWriter(new FileWriter("lib/RoadsScenario.txt"));
		int area=0;
		for(int i = GIS.resultsLayer.getSizeY()-1; i>=0; i--)
		{
			for(int j=0; j<GIS.resultsLayer.getSizeX(); j++)
			{	
				if(GIS.resultsLayer.get(j, i)!=0)// && GIS.resultsLayer.get(j, i) > 20)
					{
					//if(GIS.resultsLayer.get(j,i)>1000)
					//	out.write("1000 ");
					 out.write(GIS.resultsLayer.get(j, i) + " ");
					//	area++;
				//		}else out.write("-9999 ");
					}	
				else out.write("-9999 ");
			}
			out.newLine();
	}
	System.out.println(area);
	} catch(IOException e){}
	//**/
		
return context;

	}
}